-- I2C Control

LIBRARY ieee;
USE ieee.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY iic_control IS
	PORT	(
		CLK	: IN std_logic;
		RST_N	: IN std_logic;
		SCLK	: Out std_logic;
		SDA	: InOut std_logic;
		RSTB : Out std_logic;
		SerialStart	: In std_logic;
		SerialEnd	: Out std_logic;
		ResetStart	: In std_logic; 
		ResetEnd	: Out std_logic; 
		ReadBit : In std_logic;
		PADDR	: In std_logic_vector(7 downto 0);
		PDATAIN	: In std_logic_vector(7 downto 0); 
		PDATAOUT	: Out std_logic_vector(7 downto 0);
		PDLATCH : Out std_logic ); 
END iic_control;

ARCHITECTURE structure OF iic_control IS

	signal div512		: std_logic;
	signal SDA_dly0	: std_logic;
	signal SDA_dly1	: std_logic;
	signal SeeAck	: std_logic;
	signal SeeAck_dly0	: std_logic;
	signal SeeAck_dly1	: std_logic;
	signal counter9b_dly0	: std_logic;
	signal counter9b_dly1	: std_logic;
	signal SerialStart_dly0	: std_logic;
	signal SerialStart_dly1	: std_logic;
	signal SerialStart_edge	: std_logic;
	signal ResetStart_dly0	: std_logic;
	signal ResetStart_dly1	: std_logic;
	signal ResetStart_edge	: std_logic;
	signal ResetCount 	: std_logic_vector(6 downto 0);
	signal TransferCount 	: std_logic_vector(6 downto 0);
	signal counter9b	 	: std_logic_vector(8 downto 0);
	signal ShiftOut	 	: std_logic_vector(7 downto 0);
	signal ShiftIn	 	: std_logic_vector(39 downto 0);
   signal ShiftInLoad : std_logic;
   signal sclk_enb_n : std_logic;
   signal sda_enb_n : std_logic;
   
BEGIN
-------------------------------------------------------------- 
gen_SerialStart_dly : process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		SerialStart_dly1 <= '0';
		SerialStart_dly0 <= '0';
	elsif( CLK'event and CLK = '1' ) then
		SerialStart_dly1 <= SerialStart_dly0; 
		SerialStart_dly0 <= SerialStart; 
	end if;  
end process;

-------------------------------------------------------------- 
	SerialStart_edge <= SerialStart_dly0 and not SerialStart_dly1;
	
-------------------------------------------------------------- 
gen_ResetStart_dly : process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		ResetStart_dly1 <= '0';
		ResetStart_dly0 <= '0';
	elsif( CLK'event and CLK = '1' ) then
		ResetStart_dly1 <= ResetStart_dly0; 
		ResetStart_dly0 <= ResetStart; 
	end if;  
end process;

-------------------------------------------------------------- 
	ResetStart_edge <= ResetStart_dly0 and not ResetStart_dly1;
	
-------------------------------------------------------------- 
gen_ResetCount : process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		ResetCount <= (others => '1');
	elsif( CLK'event and CLK = '1' ) then
		if(ResetStart_edge = '1' and ResetCount(6) = '1') then 
			ResetCount <= (others => '0');
		elsif(ResetCount(6) = '0') then 
				ResetCount <= ResetCount + 1;
		end if;
	end if;  
end process;

-------------------------------------------------------------- 
   ResetEnd <= '1' when ResetCount = 63 else '0';
	RSTB <= ResetCount(6);
   
-------------------------------------------------------------- 
gen_ShiftInLoad : process( RST_N, CLK )  
begin 
   if( RST_N = '0' ) then
		ShiftInLoad <= '0';
   elsif( CLK'event and CLK = '1' ) then
      if(SerialStart_edge = '1') then 
         ShiftInLoad <= '1'; 
      elsif(div512 = '1') then
         ShiftInLoad <= '0';
      end if;
	end if;  
end process;

-------------------------------------------------------------- 
gen_TransferCount : process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		TransferCount <= (others => '1');
	elsif( CLK'event and CLK = '1' ) then
		if(SerialStart_edge = '1' and TransferCount(6) = '1') then 
			TransferCount <= (others => '0');
		elsif(div512 = '1' and TransferCount = 44 ) then 
		   TransferCount <= (others => '1');
		elsif(div512 = '1' and TransferCount(6) = '0') then 
			TransferCount <= TransferCount + 1;
		end if;
	end if;  
end process;

-------------------------------------------------------------- 
	sclk_enb_n <= '0' when (TransferCount >= 2 and TransferCount <= 29) and ReadBit = '0' 
	         else '0' when ((TransferCount >= 2 and TransferCount <= 20)
	              or (TransferCount >= 22 and TransferCount <= 40)) and ReadBit = '1'	
	         else '1';
	              
	sda_enb_n <= '0' when TransferCount <= 29 and ReadBit = '0'
	         else '0' when  TransferCount <= 40 and ReadBit = '1' 
	         else '1';

-------------------------------------------------------------- 
gen_counter9b : process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		counter9b <= (others => '0');
	elsif( CLK'event and CLK = '1' ) then
		counter9b <= counter9b + 1;
	end if;  
end process;

-------------------------------------------------------------- 
	div512 <= '1' when counter9b = 0 else '0';
	SerialEnd <= '1' when TransferCount = 44 and div512 = '1'
	         else '0';
 
-------------------------------------------------------------- 
gen_counter9b_dly : process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		counter9b_dly1 <= '0';
		counter9b_dly0 <= '0';
	elsif( CLK'event and CLK = '1' ) then
		counter9b_dly1 <= counter9b_dly0; 
		counter9b_dly0 <= counter9b(8); 
	end if;  
end process;

-------------------------------------------------------------- 
   SCLK <= (sclk_enb_n or counter9b_dly1);

-------------------------------------------------------------- 
gen_SDA_dly : process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		SDA_dly1 <= '1';
		SDA_dly0 <= '1';
	elsif( CLK'event and CLK = '1' ) then
		SDA_dly1 <= SDA_dly0; 
		SDA_dly0 <= ShiftIn(39); 
	end if;  
end process;

-------------------------------------------------------------- 
gen_SeeAck_dly : process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		SeeAck_dly1 <= '0';
		SeeAck_dly0 <= '0';
	elsif( CLK'event and CLK = '1' ) then
		SeeAck_dly1 <= SeeAck_dly0; 
		SeeAck_dly0 <= SeeAck; 
	end if;  
end process;

-------------------------------------------------------------- 
   SeeAck <= '1' when (TransferCount = 10 or TransferCount = 19 or TransferCount = 28) and ReadBit = '0' 
     else '1' when (TransferCount = 10 or TransferCount = 19 
              or (TransferCount >= 30 and TransferCount <= 38)) and ReadBit = '1'
     else '0'; 
     
   SDA <= 'Z' when SeeAck_dly1 = '1' else SDA_dly1; 

-------------------------------------------------------------- 
gen_ShiftIn : process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		ShiftIn <= (others => '1');
	elsif( CLK'event and CLK = '1' ) then
		if(ShiftInLoad = '1') then
		   if(ReadBit = '0') then 
--         	ShiftIn <= "0011110000" & PADDR & "0" & PDATAIN & "00"
         	ShiftIn <= "0010000100" & PADDR & "0" & PDATAIN & "00" -- for omnivision
   			             & "11111111111";
   		else
--         	ShiftIn <= "0011110000" & PADDR & "01" 
				ShiftIn <= "0010000100" & PADDR & "01" -- for omnivision
--         			          & "0011110010" & PDATAIN & "10";
         			          & "0010000110" & PDATAIN & "10";
 			end if;
		elsif(div512 = '1' and sda_enb_n = '0') then
			ShiftIn <= ShiftIn(38 downto 0) & '1';
		end if;
	end if;  
end process;

-------------------------------------------------------------- 
gen_ShiftOut : process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		ShiftOut <= (others => '1');
	elsif( CLK'event and CLK = '1' ) then
		if(div512 = '1') then
		   if( ReadBit = '1' and TransferCount >= 31 and TransferCount <= 38 ) then
			   ShiftOut <= ShiftOut(6 downto 0) & SDA;
			else 
			   ShiftOut <= (others => '1');
			end if;
		end if;
	end if;  
end process;

-------------------------------------------------------------- 
   PDLATCH <= '1' when ReadBit = '1' and TransferCount = 39 and div512 = '1' else '0';
   PDATAOUT <= ShiftOut;

END structure;
