-- Register Control
library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_arith.all;

entity reg_control is
   Port (
		CLK	: IN std_logic;
		RST_N	: IN std_logic;
		UA	: IN std_logic_vector(21 downto 0); 
		UD	: INOUT std_logic_vector(15 downto 0); 
		nCS	: IN std_logic;
		nWRL	: IN std_logic;
		nWRH	: IN std_logic;
		nRD	: IN std_logic;
		CapStart	: Out std_logic;
		CapEnd	: In std_logic;
		PlayMode	:	out	std_logic;		--mu0517
--		CapFrame	: Out std_logic_vector(1 downto 0);
--		PlayFrame	: Out std_logic_vector(1 downto 0);
		PADDR	: Out std_logic_vector(7 downto 0); 
		PDATAIN	: Out std_logic_vector(7 downto 0); 
		PDATAOUT	: In std_logic_vector(7 downto 0);
		PDLATCH : In std_logic; 
		SerialStart	: Out std_logic;
		SerialEnd	: In std_logic;
		ResetStart	: Out std_logic; 
		ResetEnd	: In std_logic; 
		ReadBit : Out std_logic;
		TestMode : Out std_logic_vector(1 downto 0);
		CCS : Out std_logic;
--		DelayLed0 : out std_logic_vector(7 downto 0);
--		DelayLed1 : out std_logic_vector(7 downto 0);
--		AutoLed0 : in std_logic;
--		AutoLed1 : in std_logic;
		SramAccessEnb : out std_logic
--	   SubtEnb	: Out std_logic;
--      SubtFrame	: Out std_logic_vector(1 downto 0)
		);
end reg_control;

architecture structure of reg_control is

  signal reg0_l, reg0_h : std_logic_vector( 7 downto 0 );
  signal reg1_l, reg1_h : std_logic_vector( 7 downto 0 );
  signal reg2_l, reg2_h : std_logic_vector( 7 downto 0 );
  signal reg3_l, reg3_h : std_logic_vector( 7 downto 0 );
  signal reg4_l, reg4_h : std_logic_vector( 7 downto 0 );
  signal reg5_l, reg5_h : std_logic_vector( 7 downto 0 );
  signal cs0, cs1, cs2, cs3, cs4, cs5 : std_logic;
  signal RegCs : std_logic;

begin -- architecture FM_REG_CTRL
-------------------------------------------------------------------
  RegCs <= '1' when nCS = '0' and UA(21 downto 18) = "0100" else '0';
  cs0 <= '1' when RegCs = '1' and UA(17 downto 0) = "000000000000000000" else '0';
  cs1 <= '1' when RegCs = '1' and UA(17 downto 0) = "000000000000000010" else '0';
  cs2 <= '1' when RegCs = '1' and UA(17 downto 0) = "000000000000000100" else '0';
  cs3 <= '1' when RegCs = '1' and UA(17 downto 0) = "000000000000000110" else '0';
  cs4 <= '1' when RegCs = '1' and UA(17 downto 0) = "000000000000001000" else '0';
  cs5 <= '1' when RegCs = '1' and UA(17 downto 0) = "000000000000001010" else '0';

-------------------------------------------------------------------
gen_reg0_l : process( RST_N, CLK ) 
begin
  if(RST_N = '0') then
    reg0_l <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    if(CapEnd = '1') then
		reg0_l(0) <= '0';
	 elsif( cs0 = '1' and nWRL = '0' ) then 
		reg0_l(7 downto 0) <= UD(7 downto 0);
	 end if;
  end if;
end process;

-------------------------------------------------------------------
gen_reg1_l : process( RST_N, CLK ) 
begin
  if(RST_N = '0') then
    reg1_l <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    if(SerialEnd = '1') then
      reg1_l(0) <= '0';
    elsif( cs1 = '1' and nWRL = '0' ) then 
      reg1_l(7 downto 0) <= UD(7 downto 0);
    end if;
  end if;
end process;

-------------------------------------------------------------------
gen_reg2_l : process( RST_N, CLK ) 
begin
  if(RST_N = '0') then
    reg2_l <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    if( cs2 = '1' and nWRL = '0' ) then 
      reg2_l(7 downto 0) <= UD(7 downto 0);
    end if;
  end if;
end process;

-------------------------------------------------------------------
gen_reg3_l : process( RST_N, CLK ) 
begin
  if(RST_N = '0') then
    reg3_l <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    if(ResetEnd = '1') then
      reg3_l(0) <= '0';
    elsif( cs3 = '1' and nWRL = '0' ) then 
      reg3_l(7 downto 0) <= UD(7 downto 0);
    end if;
  end if;
end process;

-------------------------------------------------------------------
gen_reg4_l : process( RST_N, CLK ) 
begin
  if(RST_N = '0') then
    reg4_l <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    if( cs4 = '1' and nWRL = '0' ) then 
      reg4_l(7 downto 0) <= UD(7 downto 0);
    end if;
  end if;
end process;

-------------------------------------------------------------------
gen_reg5_l : process( RST_N, CLK ) 
begin
  if(RST_N = '0') then
    reg5_l <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    if( cs5 = '1' and nWRL = '0' ) then 
      reg5_l(7 downto 0) <= UD(7 downto 0);
    end if;
  end if;
end process;

-------------------------------------------------------------------
gen_reg0_h : process( RST_N, CLK ) 
begin
  if(RST_N = '0') then
    reg0_h <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    if( cs0 = '1' and nWRH = '0' ) then 
      reg0_h(7 downto 0) <= UD(15 downto 8);
    end if;
  end if;
end process;

-------------------------------------------------------------------
gen_reg1_h : process( RST_N, CLK ) 
begin
  if(RST_N = '0') then
    reg1_h <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    if( cs1 = '1' and nWRH = '0' ) then 
      reg1_h(7 downto 0) <= UD(15 downto 8);
    end if;
  end if;
end process;

-------------------------------------------------------------------
gen_reg2_h : process( RST_N, CLK ) 
begin
  if(RST_N = '0') then
    reg2_h <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    if( PDLATCH = '1' ) then 
      reg2_h(7 downto 0) <= PDATAOUT(7 downto 0);
    elsif( cs2 = '1' and nWRH = '0' ) then 
      reg2_h(7 downto 0) <= UD(15 downto 8);
    end if;
  end if;
end process;

-------------------------------------------------------------------
gen_reg3_h : process( RST_N, CLK ) 
begin
  if(RST_N = '0') then
    reg3_h <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    if( cs3 = '1' and nWRH = '0' ) then 
      reg3_h(7 downto 0) <= UD(15 downto 8);
    end if;
  end if;
end process;

-------------------------------------------------------------------
gen_reg4_h : process( RST_N, CLK ) 
begin
  if(RST_N = '0') then
    reg4_h <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    if( cs4 = '1' and nWRH = '0' ) then 
      reg4_h(7 downto 0) <= UD(15 downto 8);
    end if;
  end if;
end process;

-------------------------------------------------------------------
gen_reg5_h : process( RST_N, CLK ) 
begin
  if(RST_N = '0') then
    reg5_h <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    if( cs5 = '1' and nWRH = '0' ) then 
      reg5_h(7 downto 0) <= UD(15 downto 8);
    end if;
  end if;
end process;

-------------------------------------------------------------------
   UD <=   reg0_h & reg0_l when cs0 = '1' and nRD = '0' 
      else reg1_h & reg1_l when cs1 = '1' and nRD = '0' 
      else reg2_h & reg2_l when cs2 = '1' and nRD = '0' 
      else reg3_h & reg3_l when cs3 = '1' and nRD = '0' 
      else reg4_h & reg4_l when cs4 = '1' and nRD = '0' 
      else (others => 'Z');

-------------------------------------------------------------------
   CapStart <= reg0_l(0);
	PlayMode	<=	reg0_h(7);
--   CapFrame <= reg0_l(2 downto 1); 
--   PlayFrame <= reg0_l(4 downto 3); 
   
--   SubtEnb <= reg0_h(0); -- 2007/08/01
--   SubtFrame <= reg0_h(2 downto 1); 

   SerialStart <= reg1_l(0); 
   ReadBit <= reg1_l(1); 

   PADDR	<= reg2_l(7 downto 0); 
   PDATAIN	<= reg2_h(7 downto 0); 

   ResetStart <= reg3_l(0); 
   TestMode <= reg3_l(3 downto 2); 
   CCS <= reg3_l(4); 
   SramAccessEnb <= reg3_l(5); 
   
--   DelayLed0 <= reg5_l;
--   DelayLed1 <= reg5_h;
   
end;
