-- SRAM Read/Write Control and uCom Interface --

library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_arith.all;
  use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity sram_control is
	PORT	(
		clk	: IN std_logic;
		reset_n	: IN std_logic;
		CVSYNC	: IN std_logic;
		CHSYNC	: IN std_logic;
		PCLK	: IN std_logic;
		CapStart	: IN std_logic;
		CapEnd	: OUT std_logic;
		PlayMode	:	in	std_logic;
		CD	: IN std_logic_vector(7 downto 0);
		MCLK : OUT std_logic;
		LR	: OUT std_logic_vector(7 downto 0);
		LG	: OUT std_logic_vector(7 downto 0);
		LB	: OUT std_logic_vector(7 downto 0);
		nLBLANK : OUT std_logic;
		LCLK : OUT std_logic;
		LVSYNC : OUT std_logic;
		LHSYNC : OUT std_logic;
		MD	: INOUT std_logic_vector(15 downto 0);
		nMWE	: OUT std_logic;
		nMRD	: OUT std_logic;
		nMCS0	: OUT std_logic;
		nMCS1	: OUT std_logic;
		nMCS2	: OUT std_logic;
		nMCS3	: OUT std_logic;
		nMLB	: OUT std_logic;
		nMHB	: OUT std_logic;
		MA	: OUT std_logic_vector(17 downto 0);
		UA	: IN std_logic_vector(21 downto 0);
		UD	: INOUT std_logic_vector(15 downto 0);
		nCS	: IN std_logic;
		nWRL	: IN std_logic;
		nWRH	: IN std_logic;
		nRD	: IN std_logic;
		SramAccessEnb : in std_logic
		);
end sram_control ;

architecture structure of sram_control is

component timing_gen
	Port (
		sclk	: IN std_logic;
		reset_n	: IN std_logic;
		CVSYNC	: IN std_logic;
		CHSYNC	: IN std_logic;
		PCLK	: IN std_logic;
		MCLK	: OUT std_logic;
		CapStart	: IN std_logic;
		CapEnd : OUT std_logic;
		CapBusy : OUT std_logic;
		CamWriteAddr	: OUT std_logic_vector(19 downto 0);
		CamWriteWin	: OUT std_logic;
		CVSYNC_edge	: OUT std_logic;
		nLBLANK : OUT std_logic;
		LCLK : OUT std_logic;
		LVSYNC : OUT std_logic;
		LHSYNC : OUT std_logic;
		pclkcnt	:	out	std_logic_vector(1 downto 0)
		);
end component;

signal CapBusy : std_logic;
signal MCLK_sig : std_logic;
signal CamWriteAddr : std_logic_vector(19 downto 0);
signal CamWriteWin : std_logic;
signal CamWriteEnb_N : std_logic;
signal VgaReadEnb_N : std_logic;
signal CamWriteCs0 : std_logic;
signal CamWriteCs1 : std_logic;
signal CamWriteCs2 : std_logic;
signal CamWriteCs3 : std_logic;
signal UpCs0 : std_logic;
signal UpCs1 : std_logic;
signal UpCs2 : std_logic;
signal UpCs3 : std_logic;
signal MD_RD : std_logic_vector(15 downto 0);
signal MD_CAMRD : std_logic_vector(15 downto 0);
signal UpWriteEnb_N : std_logic;
signal UpReadEnb_N : std_logic;
signal SramCs : std_logic;
signal SramCs_dly : std_logic;
signal nCS_dly : std_logic;
signal UA_dly	: std_logic_vector(20 downto 0);
signal UD_dly	: std_logic_vector(15 downto 0);
signal MD_dly	: std_logic_vector(15 downto 0);
signal WR_dly0, WR_dly1, WR_edge : std_logic;
signal Counter_WR	: std_logic_vector(1 downto 0);
signal WR_gate	: std_logic;
signal RD_dly0, RD_dly1, RD_edge : std_logic;
signal Counter_RD	: std_logic_vector(1 downto 0);
signal RD_gate	: std_logic;
signal RD_gate2	: std_logic;
signal CVSYNC_edge_sig	: std_logic;
--signal CHSYNC_edge	:	std_logic;
signal pclkcnt	:	std_logic_vector(1 downto 0);
signal gg8_pre	: std_logic_vector(7 downto 0);
signal gg8	: std_logic_vector(7 downto 0);
signal rb8	: std_logic_vector(7 downto 0);
signal grgb16	:	std_logic_vector(15 downto 0);

------------------------------------------------------------
begin
------------------------------------------------------------
timing_gen_1 : timing_gen port map (
		sclk	=> clk,
		reset_n	=> reset_n,
		CVSYNC	=> CVSYNC,
		CHSYNC	=> CHSYNC,
		PCLK	=> PCLK,
		MCLK	=> MCLK_sig, 
		CapStart	=> CapStart, 
		CapEnd	=> CapEnd, 
		CapBusy	=> CapBusy, 
		CamWriteAddr	=> CamWriteAddr, 
		CamWriteWin	=> CamWriteWin,
		CVSYNC_edge => CVSYNC_edge_sig, 
		nLBLANK	=>	nLBLANK,
		LCLK		=>	LCLK,
		LVSYNC	=>	LVSYNC,
		LHSYNC	=>	LHSYNC,
		pclkcnt	=> pclkcnt
		);

-------------------------------------------------------------- 
SramCs <= '1' when nCS = '0' and UA(21) = '1' else '0';

-------------------------------------------------------------- 
MD <= grgb16 when CamWriteEnb_N = '0' and SramAccessEnb = '0' and PlayMode = '0' else 	--mu0517
         UD_dly when UpWriteEnb_N = '0' and SramAccessEnb = '1' and PlayMode = '0'
			else (others => 'Z');
   
-------------------------------------------------------------- 
UD <= MD_dly when SramCs_dly = '1' and SramAccessEnb = '1' and nRD = '0' else (others => 'Z');
-------------------------------------------------------------- 
 
gen_gg8_pre : process( reset_n, PCLK )  
begin 
	if( reset_n = '0' ) then
		gg8_pre <= (others => '0');
	elsif( PCLK'event and PCLK = '1' ) then
	   if( pclkcnt(0) = '0' ) then		-- even number
		   gg8_pre <= CD;
	   end if;
	end if;  
end process;

-------------------------------------------------------------- 
gen_gg8 : process( reset_n, PCLK )  
begin 
	if( reset_n = '0' ) then
		gg8 <= (others => '0');
	elsif( PCLK'event and PCLK = '1' ) then
	   if( pclkcnt(0) = '1' ) then	-- odd number
		   gg8 <= gg8_pre;
	   end if;
	end if;  
end process;

-------------------------------------------------------------- 
gen_rb8 : process( reset_n, PCLK )  
begin 
	if( reset_n = '0' ) then
		rb8 <= (others => '0');
	elsif( PCLK'event and PCLK = '1' ) then
	   if( pclkcnt(0) = '1' ) then	-- odd number
		   rb8 <= CD;
	   end if;
	end if;  
end process;


gen_grgb16 : process( reset_n, PCLK )  
begin 
	if( reset_n = '0' ) then
		grgb16 <= (others => '0');
	elsif( PCLK'event and PCLK = '1' ) then
		grgb16 <= gg8 & rb8;
	end if;  
end process;
-------------------------------------------------------------- 

gen_MD_RD : process( reset_n, PCLK )
begin 
	if( reset_n = '0' ) then
		MD_RD <= (others => '0');
	elsif( PCLK'event and PCLK = '0' ) then
		if(PlayMode = '1') then
			MD_RD <= MD;
		else
			MD_RD <= grgb16;
		end if;
	end if;  
end process;

-------------------------------------------------------------- 
gen_MD_dly : process( reset_n, PCLK )
begin 
	if( reset_n = '0' ) then
		MD_dly <= (others => '0');
	elsif( PCLK'event and PCLK = '0' ) then
	   if(RD_gate2 = '1') then
         MD_dly <= MD;
	   end if;
	end if;  
end process;

grgb2rgb : process(reset_n, PCLK)
begin
	if(reset_n = '0') then
		LR <= (others=>'0');
		LG <= (others=>'0');
		LB <= (others=>'0');
	elsif(PCLK'event and PCLK='1') then
		if(pclkcnt="10") then
			LG<=MD_RD(15 downto 8);
			LR<=MD_RD(7  downto 0);
		elsif(pclkcnt="00") then
			LG<=MD_RD(15 downto 8);
			LB<=MD_RD(7  downto 0);
		end if;
	end if;
end process;

CamWriteEnb_N <= not CamWriteWin;
   
-------------------------------------------------------------- 
UpWriteEnb_N <= '0' when WR_gate = '1' and SramCs_dly = '1' else '1';
 
UpReadEnb_N <= '0' when nRD='0' and SramCs_dly = '1' else '1';
      
--------------------------------------------------------------
process(reset_n, PCLK)
begin
	if(reset_n = '0') then
		nMWE <= '0';
	elsif(PCLK'event and PCLK='0') then
		if(MCLK_sig='1' and CamWriteEnb_N='0') then
			nMWE <= '0';
		else
			nMWE <= '1';
		end if;
	end if;
end process;
            
--------------------------------------------------------------
nMRD <= '0' when UpReadEnb_N = '0' else
   	  '0' when PlayMode='1' else '1';

MA <= CamWriteAddr(17 downto 0) when (CamWriteEnb_N='0' or PlayMode='1')
			else UA_dly(17 downto 0);

-------------------------------------------------------------- 
CamWriteCs0 <= '1' when CamWriteAddr(19 downto 18) = "00" else '0';
CamWriteCs1 <= '1' when CamWriteAddr(19 downto 18) = "01" else '0';
CamWriteCs2 <= '1' when CamWriteAddr(19 downto 18) = "10" else '0';
CamWriteCs3 <= '1' when CamWriteAddr(19 downto 18) = "11" else '0';

-------------------------------------------------------------- 
UpCs0 <= '1' when UA_dly(19 downto 18) = "00" else '0';
UpCs1 <= '1' when UA_dly(19 downto 18) = "01" else '0';
UpCs2 <= '1' when UA_dly(19 downto 18) = "10" else '0';
UpCs3 <= '1' when UA_dly(19 downto 18) = "11" else '0';

-------------------------------------------------------------- 
nMCS0 <= '0' when (CamWriteCs0 = '1' and SramCs_dly = '0') 
             or (UpCs0 = '1' and SramCs_dly = '1') else '1';
nMCS1 <= '0' when (CamWriteCs1 = '1' and SramCs_dly = '0') 
             or (UpCs1 = '1' and SramCs_dly = '1') else '1';
nMCS2 <= '0' when (CamWriteCs2 = '1' and SramCs_dly = '0') 
             or (UpCs2 = '1' and SramCs_dly = '1')	else '1'; 
nMCS3 <= '0' when (CamWriteCs3 = '1' and SramCs_dly = '0') 
             or (UpCs3 = '1' and SramCs_dly = '1') else '1';

-------------------------------------------------------------- 
MCLK <= MCLK_sig;
nMLB <= '0';
nMHB <= '0';

-------------------------------------------------------------- 
gen_SramCs_dly : process( reset_n, PCLK )
begin 
	if( reset_n = '0' ) then
		SramCs_dly <= '0';
	elsif( PCLK'event and PCLK = '1' ) then
	   		SramCs_dly <= SramCs;
	end if;  
end process;

-------------------------------------------------------------- 
gen_nCS_dly : process( reset_n, PCLK )  
begin 
	if( reset_n = '0' ) then
		nCS_dly <= '0';
	elsif( PCLK'event and PCLK = '1' ) then
      		nCS_dly <= nCS;
	end if;  
end process;

-------------------------------------------------------------- 
gen_WR_dly : process( reset_n, PCLK )  
begin 
	if( reset_n = '0' ) then
		WR_dly1 <= '0';
		WR_dly0 <= '0';
	elsif( PCLK'event and PCLK = '1' ) then
   		WR_dly1 <= WR_dly0;
   		WR_dly0 <= not nWRL;
	end if;  
end process;

-------------------------------------------------------------- 
WR_edge <= WR_dly0 and not WR_dly1;

-------------------------------------------------------------- 
gen_Counter_WR : process( reset_n, PCLK )  
begin 
	if( reset_n = '0' ) then
		Counter_WR <= (others => '0');
	elsif( PCLK'event and PCLK = '1' ) then
	   if( nWRL = '1' ) then 
	      Counter_WR <= (others => '0');
	   elsif( MCLK_sig = '1' and Counter_WR /= 3 ) then 
         Counter_WR <= Counter_WR + 1; 
      end if; 
	end if;  
end process;
   
-------------------------------------------------------------- 
WR_gate <= '1' when Counter_WR = 2 and MCLK_sig = '0' else '0';

-------------------------------------------------------------- 
gen_RD_dly : process( reset_n, PCLK )
begin 
	if( reset_n = '0' ) then
		RD_dly1 <= '0';
		RD_dly0 <= '0';
	elsif( PCLK'event and PCLK = '1' ) then
   		RD_dly1 <= RD_dly0;
   		RD_dly0 <= not nRD;
	end if;  
end process;

-------------------------------------------------------------- 
RD_edge <= not nRD and not RD_dly0;

-------------------------------------------------------------- 
gen_Counter_RD : process( reset_n, PCLK ) 
begin 
	if( reset_n = '0' ) then
		Counter_RD <= (others => '0');
	elsif( PCLK'event and PCLK = '1' ) then
	   if( nRD = '1' ) then 
	      Counter_RD <= (others => '0');
	   elsif( MCLK_sig = '1' and Counter_RD /= 3 ) then 
         Counter_RD <= Counter_RD + 1; 
      end if; 
	end if;  
end process;
   
-------------------------------------------------------------- 
RD_gate  <= '1' when Counter_RD = 2 and MCLK_sig = '0' else '0';
RD_gate2 <= '1' when Counter_RD = 2 and MCLK_sig = '1' else '0';

-------------------------------------------------------------- 
gen_UA_dly : process( reset_n, PCLK ) 
begin 
	if( reset_n = '0' ) then
		UA_dly <= (others => '0');
	elsif( PCLK'event and PCLK = '1' ) then
	   		UA_dly <= UA(21 downto 1);
	end if;  
end process;

-------------------------------------------------------------- 
gen_UD_dly : process( reset_n, PCLK )
begin 
	if( reset_n = '0' ) then
		UD_dly <= (others => '0');
	elsif( PCLK'event and PCLK = '1' ) then
	   		UD_dly <= UD;
	end if;  
end process;

end structure ;
