library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity sram_model is
    Port ( n_wr : in std_logic;
           n_oe : in std_logic;
			  n_cs : in std_logic;
			  n_ble : in std_logic;
			  n_bhe : in std_logic;
           m_address : in std_logic_vector(17 downto 0);
           mdata : inout std_logic_vector(15 downto 0));
end sram_model;

architecture Behavioral of sram_model is

subtype RAMWORD is std_logic_vector(15 downto 0);
--type RAMARRAY is array( 0 to 2**18-1) of RAMWORD;
type RAMARRAY is array( 0 to 2**8-1) of RAMWORD;
signal RAMDATA : RAMARRAY;
signal ADR_IN : integer range 0 to 2**8-1;	--2**18 - 1;
signal mdata_L : std_logic_vector(7 downto 0);
signal mdata_H : std_logic_vector(7 downto 0);
signal r_n_ble : std_logic;
signal r_n_bhe : std_logic;

begin
	ADR_IN <= CONV_INTEGER(m_address(7 downto 0));

m_write:	process(n_wr) begin
		if(n_wr'event and n_wr='0') then
			if(n_cs='0') then
				if(n_ble='0' and n_bhe='0') then
					RAMDATA(ADR_IN) <= mdata;
				elsif(n_ble='0' and n_bhe='1' ) then
					RAMDATA(ADR_IN) <= RAMDATA(ADR_IN)(15 downto 8) & mdata(7 downto 0);
				elsif(n_ble='1' and n_bhe='0') then
					RAMDATA(ADR_IN) <= mdata(15 downto 8) & RAMDATA(ADR_IN)(7 downto 0);
				end if;
			end if;
		end if;
	end process m_write;

   mdata_L <= RAMDATA(ADR_IN)(7 downto 0);
	mdata_H <= RAMDATA(ADR_IN)(15 downto 8);
	r_n_ble <= n_ble when (n_cs='0' and n_wr='1' and n_oe='0') else '1';
	r_n_bhe <= n_bhe when (n_cs='0' and n_wr='1' and n_oe='0') else '1';
	mdata <= RAMDATA(ADR_IN) when (r_n_ble='0' and r_n_bhe='0') else ("ZZZZZZZZ" & mdata_L) when r_n_ble='0' else (mdata_H & "ZZZZZZZZ") when r_n_bhe='0' else (others =>'Z');

end Behavioral;
