library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity top_level is
------------------------ Conditional Gerarate -------------------
generic(
	VGA_COLORBAR		: boolean := false
	;
	VGA_CAM_GRGB		: boolean := false
	;
	VGA_CAM_RGB565		: boolean := false
	;
	VGA_CAM_BAYERRAW	: boolean := false
	;
	VGA_SRAM				: boolean := true
);
-----------------------------------------------------------------	
    Port ( y : in  STD_LOGIC_VECTOR (7 downto 0);
      n_reset : in  STD_LOGIC;
		xclk	:	in	std_logic;						-- main xtal 48MHz
		CVSYNC	: IN std_logic;					-- VSYNC from Cam
		CHSYNC	: IN std_logic;					-- HVSYNC from Cam
		PCLK	: IN std_logic;						-- Picture clock from Cam
		SCLK	: Out std_logic;						-- I2C Clock to Cam
		SDA	: InOut std_logic; 					-- I2C Data to/from Cam
		RSTB	: Out std_logic; 						-- RESET to Cam
		LR	: OUT std_logic_vector(7 downto 0);	-- Red data to DAC
		LG	: OUT std_logic_vector(7 downto 0);	-- Green data to DAC
		LB	: OUT std_logic_vector(7 downto 0);	-- Blue data to DAC
		nLBLANK : OUT std_logic;					-- Blank signal to DAC
		LCLK : OUT std_logic;						-- Clock to DAC (24MHz)
		LVSYNC : OUT std_logic;						-- VSYNC to VGA
		LHSYNC : OUT std_logic;						-- HSYNC to VGA
      rec : in  STD_LOGIC;
		MCLK :		out	std_logic;				-- Clock to Cam
      mdata : inout  STD_LOGIC_VECTOR (15 downto 0);
		n_cs0 : out  STD_LOGIC;
      n_cs1 : out  STD_LOGIC;
		n_cs2	:	out	std_logic;
		n_cs3	:	out	std_logic;
      n_oe : out  STD_LOGIC;
      n_wr : out  STD_LOGIC;
      m_address : out  STD_LOGIC_VECTOR (17 downto 0);
      n_ble : out  STD_LOGIC;
      n_bhe : out  STD_LOGIC;
		rec_led	:	out	std_logic;
		PA10	:	IN	std_logic;						-- not use (nCS0)
		nCS1	:	IN std_logic;						-- Chip select 1 from uP / SRAM in SH board
		nCS2	: IN std_logic;						-- Chip select 0 from uP
		UA	: IN std_logic_vector(21 downto 0); 	-- Address from uP
		UD	: INOUT std_logic_vector(15 downto 0);	-- Data from/to uP
		nWRL	: IN std_logic;						-- Lower Byte Enable from uP
		nWRH	: IN std_logic;						-- Upper Byte Enable from uP
		nRD	: IN std_logic;						-- Read Enable from uP
		SD_CD	:	IN		std_logic;					-- CD_SW of SD-Card
		SD_CLK	:	OUT	std_logic;				-- CLK of SD-Card
		SD_CMD	:	OUT	std_logic;				-- Din of SD-Card
		SD_DAT0	:	IN		std_logic;				-- Dout of SD-Card
		SD_DAT1	:	IN		std_logic;				-- not use
		SD_DAT2	:	IN		std_logic;				-- not use
		SD_DAT3	:	OUT	std_logic;				-- CS of SD-Card
		SD_WP	:	IN		std_logic;					-- WP of SD-Card
		PA15	:	OUT	std_logic;					-- WP of SD-Card to up
		PA7	:	IN		std_logic;					-- CS of SD-Card to up
		PB3	:	IN		std_logic;					-- Din of SD-Card to up
		PB4	:	IN		std_logic;					-- CLK of SD-Card to up
		PB5	:	OUT	std_logic;					-- Dout of SD-Card to up
		PB2	:	OUT	std_logic;					-- CD_SW of SD-Card to up
		dummy	:	out std_logic
			  );
end top_level;

architecture Behavioral of top_level is
-------------- constants -----------------------
-------------- signal declarations -------------
signal	rec_on	:	std_logic;
signal	clk_ig, clock , clock90	:	std_logic;
signal xclk_b	:	std_logic;
signal PADDR_sig : std_logic_vector(7 downto 0);
signal PDATAIN_sig : std_logic_vector(7 downto 0);
signal PDATAOUT_sig : std_logic_vector(7 downto 0);
signal PDLATCH_sig : std_logic;
signal SerialStart_sig : std_logic;
signal SerialEnd_sig : std_logic;
signal ResetStart_sig : std_logic;
signal ResetEnd_sig : std_logic;
signal ReadBit_sig : std_logic;
signal h_counter : std_logic_vector(9 downto 0);
signal v_counter : std_logic_vector(9 downto 0);
signal hs,vs,disp: std_logic;
signal rgb       : std_logic_vector(2 downto 0);
signal pclkcnt	:	std_logic_vector(1 downto 0);
signal gnd       : std_logic;
signal clk00buf  : std_logic;
signal clk00dcm  : std_logic;
signal clk90dcm	:	std_logic;
signal clkdvdcm	:	std_logic;
signal clockdv	:	std_logic;
signal bh_counter	:	std_logic_vector(15 downto 0);
signal first_byte	:	std_logic_vector(7 downto 0);
signal second_byte	:	std_logic_vector(7 downto 0);
signal line_counter	:	std_logic_vector(9 downto 0);
signal grgb16	:	std_logic_vector(15 downto 0);
signal CVSYNC_edge	: std_logic;
signal CHSYNC_edge	: std_logic;
signal CVSYNC_dly0	: std_logic;
signal CVSYNC_dly1	: std_logic;
signal CHSYNC_dly0	: std_logic;
signal CHSYNC_dly1	: std_logic;
signal pixcount	: std_logic_vector(10 downto 0);
signal gg8_pre	: std_logic_vector(7 downto 0);
signal gg8	: std_logic_vector(7 downto 0);
signal rb8	: std_logic_vector(7 downto 0);
signal AddOut_H	: std_logic_vector(18 downto 0);
signal Offset_H	: std_logic_vector(18 downto 0);
signal A_cnt	: std_logic_vector(19 downto 0);
signal CamWriteEnb_N : std_logic;
signal CamReadWin_raw : std_logic;
signal CamActiveWin_V : std_logic;
signal CamActiveWin_H : std_logic;
signal A_cnt_dly	: std_logic_vector(19 downto 0);
signal grgb16_dly	:	std_logic_vector(15 downto 0);
signal A_cnt_dly1	: std_logic_vector(19 downto 0);
signal A_cnt_dly2	: std_logic_vector(19 downto 0);
signal grgb16_dly1	:	std_logic_vector(15 downto 0);
signal grgb16_dly2	:	std_logic_vector(15 downto 0);
signal MD_RD : std_logic_vector(15 downto 0);
signal SramCs : std_logic;
signal SramCs_dly : std_logic;
signal SramAccessEnb	:	std_logic;
signal UA_dly	: std_logic_vector(20 downto 0);
signal UpCs0 : std_logic;
signal UpCs1 : std_logic;
signal UpCs2 : std_logic;
signal UpCs3 : std_logic;
signal LR_sig : std_logic_vector(7 downto 0);
signal LG_sig : std_logic_vector(7 downto 0);
signal LB_sig : std_logic_vector(7 downto 0);
signal LCLK_sig : std_logic;
signal nLBLANK_sig : std_logic;
signal CapStart_sig : std_logic;
signal CapEnd_sig : std_logic;
signal PlayMode_sig	:	std_logic;
-------------- component declarations -----------------
------------- primitives-----------------
-----------GCK -----------
component IBUFG
	port (
				I: in std_logic;
				O: out std_logic
			);
end component;
-- XST black box declaration
attribute box_type : string;
attribute box_type of IBUFG: component is "black_box";

component BUFG
	port (
				I: in std_logic;
				O: out std_logic
			);
end component;

-- XST black box declaration
attribute box_type of BUFG: component is "black_box";

----------------general components-------------

component reg_control
	Port (
		CLK	: IN std_logic;
		RST_N	: IN std_logic;
		UA	: IN std_logic_vector(21 downto 0); 
		UD	: INOUT std_logic_vector(15 downto 0); 
		nCS	: IN std_logic;
		nWRL	: IN std_logic;
		nWRH	: IN std_logic;
		nRD	: IN std_logic;
		CapStart	: Out std_logic;
		CapEnd	: In std_logic;
		PlayMode	:	out	std_logic;
		PADDR	: Out std_logic_vector(7 downto 0); 
		PDATAIN	: Out std_logic_vector(7 downto 0); 
		PDATAOUT	: In std_logic_vector(7 downto 0);
		PDLATCH : In std_logic; 
		SerialStart	: Out std_logic;
		SerialEnd	: In std_logic;
		ResetStart	: Out std_logic; 
		ResetEnd	: In std_logic; 
		ReadBit : Out std_logic;
		TestMode : Out std_logic_vector(1 downto 0);
		CCS : Out std_logic;
		SramAccessEnb : out std_logic
		);
end component;

component iic_control
    port (
		CLK	: IN std_logic;
		RST_N	: IN std_logic;
		SCLK	: Out std_logic;
		SDA	: InOut std_logic;
		RSTB	: Out std_logic;
		SerialStart	: In std_logic;
		SerialEnd	: Out std_logic;
		ResetStart	: In std_logic; 
		ResetEnd	: Out std_logic; 
		ReadBit : In std_logic;
		PADDR	: In std_logic_vector(7 downto 0);
		PDATAIN	: In std_logic_vector(7 downto 0); 
		PDATAOUT	: Out std_logic_vector(7 downto 0);
		PDLATCH : Out std_logic ); 
end component;

component sram_control
	PORT	(
		clk	: IN std_logic;
		reset_n	: IN std_logic;
		CVSYNC	: IN std_logic;
		CHSYNC	: IN std_logic;
		PCLK	: IN std_logic;
		CapStart	: IN std_logic;
		CapEnd	: OUT std_logic;
		PlayMode	:	in	std_logic;
		CD	: IN std_logic_vector(7 downto 0);
		MCLK : OUT std_logic;
		LR	: OUT std_logic_vector(7 downto 0);
		LG	: OUT std_logic_vector(7 downto 0);
		LB	: OUT std_logic_vector(7 downto 0);
		nLBLANK : OUT std_logic;
		LCLK : OUT std_logic;
		LVSYNC : OUT std_logic;
		LHSYNC : OUT std_logic;
		MD	: INOUT std_logic_vector(15 downto 0);
		nMWE	: OUT std_logic;
		nMRD	: OUT std_logic;
		nMCS0	: OUT std_logic;
		nMCS1	: OUT std_logic;
		nMCS2	: OUT std_logic;
		nMCS3	: OUT std_logic;
		nMLB	: OUT std_logic;
		nMHB	: OUT std_logic;
		MA	: OUT std_logic_vector(17 downto 0);
		UA	: IN std_logic_vector(21 downto 0);
		UD	: INOUT std_logic_vector(15 downto 0);
		nCS	: IN std_logic;
		nWRL	: IN std_logic;
		nWRH	: IN std_logic;
		nRD	: IN std_logic;
		SramAccessEnb : in std_logic
		);
end component;


-------------------------------------------------------
begin
--------------  component instantiaton ----------------

  gnd   <= '0';

  DCM_INST2 : DCM
    generic map (
      DLL_FREQUENCY_MODE    => "LOW",
      DUTY_CYCLE_CORRECTION => true,
		CLK_FEEDBACK => "1X"
      )
    port map (
      CLKIN    => xclk,
      CLKFB    => xclk_b,
      DSSEN    => gnd,
      PSINCDEC => gnd,
      PSEN     => gnd,
      PSCLK    => gnd,
      RST      => gnd,
      CLK0     => clk00dcm,
      CLK90    => clk90dcm,
      CLK180   => open,
      CLK270   => open,
      CLK2X    => open,
      CLK2X180 => open,
      CLKDV    => clkdvdcm,
      CLKFX    => open,
      CLKFX180 => open,
      LOCKED   => open,
      PSDONE   => open,
      STATUS   => open
      );


  BUFG_CLK1X : BUFG
    port map (
	   I  => xclk_b,
		O  => clock
		);

  BUFG_CLK00 : BUFG
    port map (
	   I  => clk00dcm,
		O  => xclk_b
		);

  BUFG_CLK90 : BUFG
    port map (
	   I  => clk90dcm,
		O  => clock90
		);

  BUFG_CLKDV : BUFG
    port map (
	   I  => clkdvdcm,
		O  => clockdv
		);


igclk0:	IBUFG
	port map (
				I	=> PCLK,
				O	=> clk_ig
				);

reg_control_1 : reg_control port map (
		CLK	=> xclk_b,
		RST_N	=> n_reset,
		UA	=> UA, 
		UD	=> UD,
		nCS	=> nCS2,
		nWRL	=> nWRL,
		nWRH	=> nWRH,
		nRD	=> nRD,
		CapStart	=> CapStart_sig,
		CapEnd	=> CapEnd_sig,
		PlayMode	=>	PlayMode_sig,
		PADDR	=> PADDR_sig,
		PDATAIN	=> PDATAIN_sig,
		PDATAOUT	=> PDATAOUT_sig,
		PDLATCH	=> PDLATCH_sig,
		SerialStart	=> SerialStart_sig,
		SerialEnd	=> SerialEnd_sig, 
		ResetStart	=> ResetStart_sig,
		ResetEnd	=> ResetEnd_sig,
		ReadBit	=> ReadBit_sig,
		CCS	=> RSTB,
		SramAccessEnb	=> SramAccessEnb 
		);

iic_control_1 : iic_control port map (
		CLK	=> xclk_b,
		RST_N	=> n_reset,
		SCLK	=> SCLK, 
		SDA	=> SDA, 
		RSTB => open,
		SerialStart	=> SerialStart_sig, 
		SerialEnd	=> SerialEnd_sig, 
		ResetStart	=> ResetStart_sig,
		ResetEnd	=> ResetEnd_sig,
		ReadBit	=> ReadBit_sig,
		PADDR	=> PADDR_sig, 
		PDATAIN	=> PDATAIN_sig,
		PDATAOUT	=> PDATAOUT_sig,
		PDLATCH	=> PDLATCH_sig );


sram_rdwr_process : if VGA_SRAM generate
sram_control_1 : sram_control port map (
		clk	=> xclk_b,
		reset_n	=> n_reset,
		CVSYNC	=> CVSYNC,
		CHSYNC	=> CHSYNC,
		PCLK		=> clk_ig,
		CapStart	=> CapStart_sig,
		CapEnd	=> CapEnd_sig,
		PlayMode	=>	PlayMode_sig,
		CD	=> y,
		MCLK	=> MCLK,
		LR	=>	LR_sig,
		LG	=> LG_sig,
		LB	=> LB_sig,
		nLBLANK	=> nLBLANK_sig,
		LCLK	=> LCLK_sig,
		LVSYNC	=> open,
		LHSYNC	=> LHSYNC,
		MD	=> mdata,
		nMWE	=> n_wr,
		nMRD	=> n_oe,
		nMCS0	=> n_cs0,
		nMCS1	=> n_cs1,
		nMCS2	=> n_cs2,
		nMCS3	=> n_cs3,
		nMLB	=> n_ble,
		nMHB	=> n_bhe,
		MA		=> m_address,
		UA		=> UA,
		UD		=> UD,
		nCS	=> nCS2,
		nWRL	=> nWRL,
		nWRH	=> nWRH,
		nRD	=> nRD,
		SramAccessEnb	=> SramAccessEnb
		);

end generate;


---------------------------------------------------
-------------- concurrent statements---------------

--------------- signal definitions-----------------
rec_on <= rec;

PB2 <= SD_CD;		-- CD_SW of SD-Card
SD_CLK <= PB4;		-- CLK of SD-Card
SD_CMD <= PB3;		-- Din of SD-Card
PB5 <= SD_DAT0;	-- Dout of SD-Card
SD_DAT3 <= PA7;	-- CS of SD-Card to up
PA15 <= SD_WP;		-- WP of SD-Card

dummy <= nCS1 and PA10 and SD_DAT1 and SD_DAT2;	-- and rec;		--nouse pin

vga_sram_process: if VGA_SRAM generate
-------------------------------------------------------------
nLBLANK <= nLBLANK_sig;
LCLK <= LCLK_sig;
LVSYNC <= not(CVSYNC);


LR <= LR_sig;
LG <= LG_sig;
LB <= LB_sig;

--rec_led <= rec_on;
--rec_led <= CapStart_sig;
rec_led <= PlayMode_sig;
-------------------------------------------------------------
end generate;


vga_process	:	if not VGA_SRAM generate
---------------------- VGA procedures -----------------------
nLBLANK<=disp;
LCLK<=clockdv;
--nLPSAVE<='1';
LHsync<=hs;
LVsync<=vs;
	

MCLK <= clockdv;
-------------------------------------------------------------	
end generate;

----------------------- VGA COLORBAR ------------------------
vga_colorbar2: if VGA_COLORBAR generate
process (clockdv)
begin
  if clockdv'event and clockdv='1' then
		if (h_counter<763) then
    		h_counter<=h_counter+1;
		else 
		   h_counter<=(others=>'0');
			if (v_counter<524) then
				v_counter<=v_counter+1;
			else
				v_counter<=(others=>'0');
			end if;
		end if;
  end if;
end process;

hs<='0' when h_counter>=640+16 and h_counter<640+16+64 else '1';
vs<='0' when v_counter>=480+10 and v_counter<480+10+2  else '1';
disp<='0' when h_counter>=640 or v_counter>=480 else '1';

rgb<=
	"111" when h_counter<80   else
	"110" when h_counter<160  else
	"011" when h_counter<240  else
	"010" when h_counter<320  else
	"101" when h_counter<400  else
	"100" when h_counter<480  else
	"001" when h_counter<560  else "000";
LR<=(others=>rgb(2));
LG<=(others=>rgb(1));
LB<=(others=>rgb(0));

m_address <= (others => '0');
n_cs0 <= '1';
n_cs1 <= '1';
n_cs2 <= '1';
n_cs3 <= '1';
n_bhe <= '1';
n_ble <= '1';
n_oe <= '1';
n_wr <= '1';
mdata <= (others => '0');
rec_led <= '0';

end generate;

---------------------- CAMERA RGB ------------------------
vga_cam_grgb1: if VGA_CAM_GRGB generate
----------------------------------------------------------

		pclk_count: process(n_reset, clock)
		begin
			if(n_reset='0') then
				pclkcnt <= (others=>'0');
			elsif(clock'event and clock='0') then
				if(CHSYNC='0') then
					pclkcnt <= (others=>'0');				--Atten! 'pclkcnt' is 2 bit width!
				else
					pclkcnt <= pclkcnt + 1;
				end if;
			end if;
		end process;

		gen_PixCount : process( n_reset, clock )  
		begin 
			if( n_reset = '0' ) then
				pixcount <= (others => '0');
			elsif( clock'event and clock = '0' ) then
				if(CHSYNC_edge = '1') then 
					pixcount <= (others => '0');			-- pixcount is 11 bit width
				else
					pixcount <= pixcount + 1;
				end if; 
			end if;  
		end process;


		--SimpleCam : process(clock, n_reset)		--camera direct to VGA
		--begin
		--	if(n_reset = '0') then
		--		LR <= (others=>'0');
		--		LG <= (others=>'0');
		--		LB <= (others=>'0');
		--	elsif(clock'event and clock='1') then
		--		if(pclkcnt(0)='0') then
		--			LG<=y;
		--		elsif(pclkcnt(1)='1') then
		--			LB<=y;
		--		elsif(pclkcnt(1)='0') then
		--			LR<=y;
		--		end if;
		--	end if;
		--end process;

		-------------------------------------------------------------------------
		--gen_cbarGRGB	:	process(n_reset, clock)			-- color bar for check --
		--begin
		--	if(n_reset = '0') then
		--		grgb16 <= (others => '0');
		--	elsif clock'event and clock='1' then
		--		if(pclkcnt="01") then			--1,5,9...GR
		--			if    pixcount<80*2 then
		--				grgb16 <= x"FFFF";
		--			elsif pixcount<160*2 then
		--				grgb16 <= x"FFFF";
		--			elsif pixcount<240*2 then
		--				grgb16 <= x"FF00";
		--			elsif pixcount <320*2 then
		--				grgb16 <= x"FF00";
		--			elsif pixcount <400*2 then
		--				grgb16 <= x"00FF";
		--			elsif pixcount <480*2 then
		--				grgb16 <= x"00FF";
		--			elsif pixcount <560*2 then
		--				grgb16 <= x"0000";
		--			else
		--				grgb16 <= x"0000";
		--			end if;
		--		elsif(pclkcnt="11") then		--3,7,11...GB
		--			if    pixcount <80*2 then
		--				grgb16 <= x"FFFF";
		--			elsif pixcount <160*2 then
		--				grgb16 <= x"FF00";
		--			elsif pixcount <240*2 then
		--				grgb16 <= x"FFFF";
		--			elsif pixcount <320*2 then
		--				grgb16 <= x"FF00";
		--			elsif pixcount <400*2 then
		--				grgb16 <= x"00FF";
		--			elsif pixcount <480*2 then
		--				grgb16 <= x"0000";
		--			elsif pixcount <560*2 then
		--				grgb16 <= x"00FF";
		--			else
		--				grgb16 <= x"0000";
		--			end if;
		--		end if;
		--	end if;
		--end process;

		-------------------------------------------------------------- 
		gen_gg8_pre : process( n_reset, clock )  
		begin 
			if( n_reset = '0' ) then
				gg8_pre <= (others => '0');
			elsif( clock'event and clock = '1' ) then
				if (pclkcnt(0) = '0') then
					gg8_pre <= y;
				end if;
			end if;  
		end process;

		-------------------------------------------------------------- 
		gen_gg8 : process( n_reset, clock )  
		begin 
			if( n_reset = '0' ) then
				gg8 <= (others => '0');
			elsif( clock'event and clock = '1' ) then
				if( pclkcnt(0) = '1' ) then
					gg8 <= gg8_pre;
				end if;
			end if;  
		end process;

		-------------------------------------------------------------- 
		gen_rb8 : process( n_reset, clock )  
		begin 
			if( n_reset = '0' ) then
				rb8 <= (others => '0');
			elsif( clock'event and clock = '1' ) then
				if( pclkcnt(0) = '1' ) then
					rb8 <= y;
				end if;
			end if;  
		end process;

		-------------------------------------------------------------- 
		gen_grgb16 : process( n_reset, clock )  
		begin 
			if( n_reset = '0' ) then
				grgb16 <= (others => '0');
			elsif( clock'event and clock = '1' ) then
				grgb16 <= gg8 & rb8;
			end if;  
		end process;

		gen_grgb16_dly : process( n_reset, clock )  
		begin 
			if( n_reset = '0' ) then
				grgb16_dly <= (others => '0');
				grgb16_dly1 <= (others => '0');
				grgb16_dly2 <= (others => '0');
			elsif( clock'event and clock = '1' ) then
				grgb16_dly <= grgb16;
				grgb16_dly1 <= grgb16_dly;
				grgb16_dly2 <= grgb16_dly1;
			end if;  
		end process;

--------------------------------------------------------------
toponly_prcess: if not VGA_SRAM generate
--------------------------------------------------------------
grgb2rgb : process(n_reset, clock)
begin
	if(n_reset = '0') then
		LR <= (others=>'0');
		LG <= (others=>'0');
		LB <= (others=>'0');
	elsif(clock'event and clock='1') then
		if(pclkcnt="00") then
			LG<=MD_RD(15 downto 8);
			LR<=MD_RD(7  downto 0);
		elsif(pclkcnt="10") then
			LG<=MD_RD(15 downto 8);
			LB<=MD_RD(7  downto 0);
		end if;
	end if;
end process;
--------------------------------------------------------------

process(n_reset, clock)
begin
	if(n_reset = '0') then
		bh_counter <= (others => '0');
	elsif clock'event and clock='1' then
		if(CHSYNC='1') then
			bh_counter <= (others => '0');
		elsif(bh_counter>784*2-1) then
			bh_counter <= (others => '0');
		else
			bh_counter <= bh_counter + 1;
		end if;
	end if;
end process;

-------------------------------------------------------------- 
gen_CVSYNC_dly : process( n_reset, clock )  
begin 
	if( n_reset = '0' ) then
		CVSYNC_dly1 <= '0';
		CVSYNC_dly0 <= '0';
	elsif( clock'event and clock = '1' ) then
		CVSYNC_dly1 <= CVSYNC_dly0; 
		CVSYNC_dly0 <= CVSYNC; 
	end if;  
end process;

-------------------------------------------------------------- 
CVSYNC_edge <= CVSYNC_dly0 and not CVSYNC_dly1;

-------------------------------------------------------------- 
gen_CHSYNC_dly : process( n_reset, clock )  
begin 
	if( n_reset = '0' ) then
		CHSYNC_dly1 <= '0';
		CHSYNC_dly0 <= '0';
	elsif( clock'event and clock = '1' ) then
		CHSYNC_dly1 <= CHSYNC_dly0; 
		CHSYNC_dly0 <= CHSYNC; 
	end if;  
end process;

-------------------------------------------------------------- 
CHSYNC_edge <= CHSYNC_dly0 and not CHSYNC_dly1;

gen_CamLineCount : process( n_reset, clock )  
begin 
	if( n_reset = '0' ) then
		line_counter <= (others => '1');
	elsif( clock'event and clock = '1' ) then
		if(CVSYNC_edge = '1') then 
			line_counter <= (others => '1');
		elsif(CHSYNC_edge = '1') then 
			line_counter <= line_counter + 1;
		end if; 
	end if;  
end process;

hs<='0' when bh_counter>=16*2 and bh_counter<(16+80)*2 else '1';
vs<=not(CVSYNC);
disp<='0' when pixcount >=640*2 or line_counter>=480 else '1';

------------------------------generate address----------------------------
  AddOut_H <= Offset_H + 640;

gen_Offset_H : process( n_reset, clock )
begin
  if( n_reset = '0' ) then
    Offset_H <= (others => '0');
  elsif( clock'event and clock = '1' ) then
    if( CVSYNC_edge = '1' ) then 
      Offset_H <= (others => '0');
    elsif( CHSYNC_edge = '1' ) then
      Offset_H <= AddOut_H;
    end if;
  end if;
end process;

gen_Addr : process( n_reset, clock )  
begin 
	if( n_reset = '0' ) then
		A_cnt <= (others => '0');
	elsif( clock'event and clock = '1' ) then
		A_cnt <= Offset_H + pixcount(10 downto 1);
	end if;  
end process;

gen_Acnt_dly : process( n_reset, clockdv)
begin
	if( n_reset = '0' ) then
		A_cnt_dly <= (others => '0');
		A_cnt_dly1 <= (others => '0');
		A_cnt_dly2 <= (others => '0');
	elsif( clockdv'event and clockdv = '0' ) then
		A_cnt_dly <= A_cnt;
		A_cnt_dly1 <= A_cnt_dly;
		A_cnt_dly2 <= A_cnt_dly1;
	end if;
end process;

----------------------------------------------------
n_cs0 <= '0' when A_cnt(19 downto 18)="00"
            or (UpCs0 = '1' and SramCs = '1') else '1';
n_cs1 <= '0' when A_cnt(19 downto 18)="01" 
            or (UpCs1 = '1' and SramCs = '1') else '1';
n_cs2 <= '0' when A_cnt(19 downto 18)="10" 
            or (UpCs2 = '1' and SramCs = '1') else '1';
n_cs3 <= '0' when A_cnt(19 downto 18)="11" 
            or (UpCs3 = '1' and SramCs = '1') else '1';

UpCs0 <= '1' when UA(19 downto 18) = "00" else '0';
UpCs1 <= '1' when UA(19 downto 18) = "01" else '0';
UpCs2 <= '1' when UA(19 downto 18) = "10" else '0';
UpCs3 <= '1' when UA(19 downto 18) = "11" else '0';

------------------------------SRAM access----------------------------
CamWriteEnb_N <= not(CamReadWin_raw and clockdv);

gen_MD_RD : process( n_reset, clock )  
begin 
	if( n_reset = '0' ) then
		MD_RD <= (others => '0');
	elsif( clock'event and clock = '0' ) then
	   if(clockdv = '0') then
	   		MD_RD <= mdata;
	   end if;
	end if;  
end process;
 
n_bhe <= '0';
n_ble <= '0';

n_oe <= '0' when rec_on='0' else '1';
--n_wr <= '1';
n_wr <= '0' when clockdv = '1' and CamWriteEnb_N = '0' and clock='0' and rec_on='1' else '1'; 

CamActiveWin_V <= '1' when line_counter >= 0 and line_counter < 480 else '0';
CamActiveWin_H <= '1' when pixcount < 640*2 else '0';

gen_CamReadWin_raw : process( n_reset, clock )
begin 
	if( n_reset = '0' ) then
		CamReadWin_raw <= '0';
	elsif( clock'event and clock = '1' ) then
         CamReadWin_raw <= CamActiveWin_V and CamActiveWin_H;  
	end if;  
end process;
------------------------ Up interface ---------------------
SramCs <= '1' when nCS2 = '0' and UA(21) = '1' else '0';


m_address <= UA(17 downto 0) when SramCs = '1' else A_cnt_dly(17 downto 0);

mdata <= grgb16 when CamWriteEnb_N = '0' and rec_on='1'
			else (others => 'Z');

UD <= mdata when SramCs = '1' and SramAccessEnb = '1' and nRD = '0' else (others => 'Z');

rec_led <= SramCs;

-------------------------------------------------------------------
end generate;			--not VGA_SRAM
-------------------------------------------------------------------
-------------------------------------------------------------------			
end generate;			--VGA_CAM_GRGB
-------------------------------------------------------------------

---------------------- CAMERA RGB565 ------------------------
vga_cam_grgb2: if VGA_CAM_RGB565 generate
-------------------------------------------------------------
pclk_count: process(n_reset, clock)
begin
	if(n_reset='0') then
		pclkcnt <= (others=>'0');
	elsif(clock'event and clock='1') then
		if(CHSYNC='0') then
			pclkcnt <= (others=>'0');
		else
			pclkcnt <= pclkcnt + 1;
		end if;
	end if;
end process;

SimpleCam : process(clock, n_reset)
begin
	if(n_reset = '0') then
		first_byte <= (others=>'0');
		second_byte <= (others=>'0');
	elsif(clock'event and clock='1') then
		if(pclkcnt(0)='0') then
			first_byte<=y;
		elsif(pclkcnt(0)='1') then
			second_byte<=y;
		end if;
	end if;
end process;

process(clock, n_reset)
begin
	if(n_reset = '0') then
		bh_counter <= (others => '0');
	elsif clock'event and clock='1' then
		if(CHSYNC='1') then
			bh_counter <= (others => '0');
		elsif(bh_counter>784*2-1) then
			bh_counter <= (others => '0');
		else
			bh_counter <= bh_counter + 1;
		end if;
	end if;
end process;

LR <= first_byte(7 downto 3) & "000";
LG <= first_byte(2 downto 0) & second_byte(7 downto 5) & "00";
LB <= second_byte(4 downto 0) & "000";

hs<='0' when bh_counter>=16*2 and bh_counter<(16+64)*2 else '1';
vs<=not(CVSYNC);
disp<='1';

m_address <= (others => '0');
n_cs0 <= '1';
n_cs1 <= '1';
n_cs2 <= '1';
n_cs3 <= '1';
n_bhe <= '1';
n_ble <= '1';
n_oe <= '1';
n_wr <= '1';
mdata <= (others => '0');
rec_led <= '0';

end generate;

---------------------- CAMERA Bayer ------------------------
vga_cam_grgb3: if VGA_CAM_BAYERRAW generate

pclk_count: process(n_reset, clock)
begin
	if(n_reset='0') then
		pclkcnt <= (others=>'0');
	elsif(clock'event and clock='1') then
		if(CHSYNC='0') then
			pclkcnt <= (others=>'0');
		else
			pclkcnt <= pclkcnt + 1;
		end if;
	end if;
end process;

line_count: process(n_reset, CHSYNC)
begin
	if(n_reset='0') then
		line_counter <= (others=>'0');
	elsif(CHSYNC'event and CHSYNC='1') then
		if(CVSYNC='1') then
			line_counter <= (others=>'0');
		else
			line_counter <= line_counter + 1;
		end if;
	end if;
end process;

SimpleCam : process(clock, n_reset)
begin
	if(n_reset = '0') then
		LR <= (others=>'0');
		LG <= (others=>'0');
		LB <= (others=>'0');
	elsif(clock'event and clock='1') then
		if(line_counter(0)='0') then
			if(pclkcnt(0)='0') then
				LG<=y;
			else
				LR<=y;
			end if;
		else
			if(pclkcnt(0)='1') then
				LG<=y;
			else
				LB<=y;
			end if;
		end if;
	end if;
end process;

process(clock, n_reset)
begin
	if(n_reset = '0') then
		bh_counter <= (others => '0');
	elsif clock'event and clock='1' then
		if(CHSYNC='1') then
			bh_counter <= (others => '0');
		else
			bh_counter <= bh_counter + 1;
		end if;
	end if;
end process;

hs<=CHSYNC;
vs<=not(CVSYNC);
disp<='1';

m_address <= (others => '0');
n_cs0 <= '1';
n_cs1 <= '1';
n_cs2 <= '1';
n_cs3 <= '1';
n_bhe <= '1';
n_ble <= '1';
n_oe <= '1';
n_wr <= '1';
mdata <= (others => '0');
rec_led <= '0';

end generate;

-----------------------end of file------------------------
end Behavioral;

