/*-----------------------------------------------------------------------*/
/* Low level disk I/O module skeleton for FatFs     (C)ChaN, 2007        */
/*-----------------------------------------------------------------------*/
/* This is a stub disk I/O module that acts as front end of the existing */
/* disk I/O modules and attach it to FatFs module with common interface. */
/*-----------------------------------------------------------------------*/
//mu0308,for SH-2/SH7144F


/* printfwb_ */
#include "cq_printf.h"			//mu0412
#include "diskio.h"
//#include "mmcdrv.h"			//mu0308

/*-----------------------------------------------------------------------*/
/* Correspondence between physical drive number and physical drive.      */
/* Note that Tiny-FatFs supports only single drive and always            */
/* accesses drive number 0.                                              */

//m	#define ATA		0
//m	#define MMC		1
//m	#define USB		2
#define MMC_TIMEOUT_ERROR       (-1)
#define MMC_LBA_RANGE_ERROR     (-2)
#define MMC_INIT_ERROR          (-3)
#define MMC_CARD_DECTECT_ERROR  (-6)
#define MMC_WRITE_PROTECT_ERROR (-7)

#define CMD58 0x7A /* get OCR register */
#define UNBCD(a) ((a >> 4) * 10   +  (a & 0x0F))
#define BCD(a)   (((a / 10) << 4) |  ((a % 10) & 0x0F))
/* MMC\PbgC^[tF[X */
#define MMC_DrverInit()	mmcio_SocketInit();

/*typedef long long                u64;
typedef unsigned long            u32;
typedef unsigned short           u16;
typedef unsigned char            u8;
typedef unsigned int    		 Int32U;   */ // Unsigned 32 bit quantity
//typedef unsigned char BOOL;
//#define FALSE	0
//#define TRUE	1
typedef unsigned int    Int32U;   // Unsigned 32 bit quantity

/*typedef struct
{u16 year; u8 month; u8 mday; u8 wday; u8 hour; u8 min; u8 sec;
}RTCX; */

extern DSTATUS Stat = STA_NOINIT;	/* Disk status mu0308 */

static BYTE CardType=2;			/* b0:MMC, b1:SDv1, b2:SDv2, b3:Block addressing */

Int32U R64CNT,RSECCNT,RMINCNT,RHRCNT,RWKCNT,RDAYCNT,RMONCNT,RYEARCNT;
Int32U RSECAR,RMINAR,RHRAR,RWKAR,RDAYAR,RMONAR,RCR1,RCR2,RCR3;

BYTE rtcYear, rtcMon, rtcMday, rtcHour, rtcMin, rtcSec;
BYTE RTC_SEC,RTC_MIN,RTC_HOUR,RTC_DOW,RTC_DOM,RTC_MONTH;
WORD RTC_YEAR;


/*-----------------------------------------------------------------------*/
/* Inidialize a Drive                                                    */

DSTATUS disk_initialize (
	BYTE drv				/* Physical drive nmuber (0..) */
)
{
//	DSTATUS stat;
	int i,result;
	
//	stat = 0;				//mu0412
//    stat |= STA_NOINIT;
    Stat |= STA_NOINIT;		//mu0412

	MMC_DrverInit();		/* MMC\PbgC^[tF[X */
	i=MMC_CardInit();		/* MMCJ[h */
	if (i) {
		cq_printf("Card Initialize Error! %d\n",i);
		return Stat;		//stat;
	}
//    stat &= ~STA_NOINIT;    /* When device goes ready, clear STA_NOINIT */
        Stat &= ~STA_NOINIT;    /* When device goes ready, clear STA_NOINIT */
	return	Stat;			//stat; 
}



/*-----------------------------------------------------------------------*/
/* Return Disk Status                                                    */

DSTATUS disk_status (
	BYTE drv		/* Physical drive nmuber (0..) */
)
{
//	DSTATUS stat;
//	int result;

	if (drv) return STA_NOINIT;		/* Supports only single drive */
//	return Stat;

	if(mmcio_WritePortectCheck()){
//		cq_printf("write protected.\n");
		Stat |= STA_PROTECT;
	} else Stat &= ~STA_PROTECT;
	if(!mmcio_CardDetectCheck()){
//		cq_printf("No disk.\n");
		Stat |= STA_NODISK;
	}
//	if(MmcDskStatus.DiskStatus == DiskCommandPass){
//		cq_printf("Initialized.\n\r");
//        Stat &= ~STA_NOINIT;    /* When device goes ready, clear STA_NOINIT */
//	}
		return Stat;
}

/*-----------------------------------------------------------------------*/
/* Read Sector(s)                                                        */

DRESULT disk_read (
	BYTE drv,		/* Physical drive nmuber (0..) */
	BYTE *buff,		/* Data buffer to store read data */
	DWORD sector,	/* Sector address (LBA) */
	BYTE count		/* Number of sectors to read (1..255) */
)
{
	DRESULT res;
	int result;


	if (drv) return RES_PARERR;

	result = MMC_Read(sector, buff, count);
	if (result==MMC_CARD_DECTECT_ERROR) return RES_NOTRDY;
	if (result==MMC_LBA_RANGE_ERROR) return RES_PARERR;
	else return RES_OK;	//RES_ERROR;
}

/*-----------------------------------------------------------------------*/
/* Write Sector(s)                                                       */

#if _READONLY == 0
DRESULT disk_write (
	BYTE drv,			/* Physical drive nmuber (0..) */
	const BYTE *buff,	/* Data to be written */
	DWORD sector,		/* Sector address (LBA) */
	BYTE count			/* Number of sectors to write (1..255) */
)
{
	DRESULT res;
	int result;

	if (drv) return RES_PARERR;
	
	result = MMC_Write(sector, buff, count);
	if (result==MMC_CARD_DECTECT_ERROR) return RES_NOTRDY;
	if (result==MMC_WRITE_PROTECT_ERROR) return RES_WRPRT;
	if (result==MMC_LBA_RANGE_ERROR) return RES_PARERR;
	else return RES_OK;		//RES_ERROR; 
}

#endif /* _READONLY */



/*-----------------------------------------------------------------------*/
/* Miscellaneous Functions                                               */

DRESULT disk_ioctl (
	BYTE drv,		/* Physical drive nmuber (0..) */
	BYTE ctrl,		/* Control code */
	void *buff		/* Buffer to send/receive control data */
)
{
	DRESULT res;
	BYTE n, csd[16], *ptr = buff;
	WORD csize;
	int result, response;
	int i;

	if (drv) return RES_PARERR;
	if (Stat & STA_NOINIT) return RES_NOTRDY;

	res = RES_ERROR;
	switch (ctrl) {
		case CTRL_SYNC :		/* Make sure that no pending write process */
//			SELECT();
//			if (wait_ready() == 0xFF)
//				res = RES_OK;
			if (mmc_busywait() == 0)
				res = RES_OK;
			break;

		case GET_SECTOR_COUNT :	/* Get number of sectors on the disk (DWORD) */
			if (!(i=MMC_GetCSD(csd))) {
//			if ((send_cmd(CMD9, 0) == 0) && rcvr_datablock(csd, 16)) {
				if ((csd[0] >> 6) == 1) {	/* SDC ver 2.00 */
					csize = csd[9] + ((WORD)csd[8] << 8) + 1;
					*(DWORD*)buff = (DWORD)csize << 10;
				} else {					/* SDC ver 1.XX or MMC */
					n = (csd[5] & 15) + ((csd[10] & 128) >> 7) + ((csd[9] & 3) << 1) + 2;
					csize = (csd[8] >> 6) + ((WORD)csd[7] << 2) + ((WORD)(csd[6] & 3) << 10) + 1;
					*(DWORD*)buff = (DWORD)csize << (n - 9);
				}
				res = RES_OK;
			} else {cq_printf(" Get Error!  error code=%Xh\n",i);
			}

			break;

		case GET_SECTOR_SIZE :	/* Get R/W sector size (WORD) */
			*(WORD*)buff = 512;
			res = RES_OK;
			break;

		case GET_BLOCK_SIZE :	/* Get erase block size in unit of sector (DWORD) */
//			if (CardType & 4) {			/* SDC ver 2.00 */
//				if (send_cmd(ACMD13, 0) == 0) {		/* Read SD status */
//					rcvr_spi();
//					if (rcvr_datablock(csd, 16)) {				/* Read partial block */
//						for (n = 64 - 16; n; n--) rcvr_spi();	/* Purge trailing data */
//						*(DWORD*)buff = 16UL << (csd[10] >> 4);
//						res = RES_OK;
//					}
//				}
//			} else {					/* SDC ver 1.XX or MMC */
//				if ((send_cmd(CMD9, 0) == 0) && rcvr_datablock(csd, 16)) {	/* Read CSD */
			if (!(MMC_GetCSD(csd))) {
					if (CardType & 2) {			/* SDC ver 1.XX */
						*(DWORD*)buff = (((csd[10] & 63) << 1) + ((WORD)(csd[11] & 128) >> 7) + 1) << ((csd[13] >> 6) - 1);
					} else {					/* MMC */
						*(DWORD*)buff = ((WORD)((csd[10] & 124) >> 2) + 1) * (((csd[11] & 3) << 3) + ((csd[11] & 224) >> 5) + 1);
					}
					res = RES_OK;
//				}
			}
			break;

		case MMC_GET_TYPE :		/* Get card type flags (1 byte) */
			*ptr = CardType;
			res = RES_OK;
			break;

		case MMC_GET_CSD :		/* Receive CSD as a data block (16 bytes) */
//			if (send_cmd(CMD9, 0) == 0		/* READ_CSD */
//				&& rcvr_datablock(ptr, 16))
			if (!(MMC_GetCSD(ptr))) {
				res = RES_OK;
			}
			break;

		case MMC_GET_CID :		/* Receive CID as a data block (16 bytes) */
//			if (send_cmd(CMD10, 0) == 0		/* READ_CID */
//				&& rcvr_datablock(ptr, 16))
			if (MMC_GetCID(ptr) == 0 )
				res = RES_OK;
			break;

		case MMC_GET_OCR :		/* Receive OCR as an R3 resp (4 bytes) */
//			if (send_cmd(CMD58, 0) == 0) {	/* READ_OCR */
//				for (n = 4; n; n--) *ptr++ = rcvr_spi();
			if (mmcio_CardDetectCheck()){
				mmcio_CsCtrl(1);		/* CS enable */
				response = mmc_commandtoken(CMD58,0,0x00);
				if (response == 0) {			/* begin receive data */
					response = mmc_recvblock(ptr, 4, 1);
				}
				/* Dummy Clock && CS = H */
				mmcio_CsCtrl(0);		/* CS disable */
				res = RES_OK;
			}
			break;

		case MMC_GET_SDSTAT :	/* Receive SD statsu as a data block (64 bytes) */
//			if (send_cmd(ACMD13, 0) == 0) {	/* SD_STATUS */
//				rcvr_spi();
//				if (rcvr_datablock(ptr, 64))
//					res = RES_OK;
//			}
			break;

		default:
			res = RES_PARERR;
	}

//	release_spi();

	return res;
}

DWORD get_fattime()
{
    DWORD tmr;
    RTCX rtc;

    /* Real Time Clock */
    rtc_gettime (&rtc);

    rtcYear = rtc.year-2028;	//-19;	//+= 100;

    rtcMon = rtc.month;	//2;	//UNBCD(RMONCNT);
    rtcMday = rtc.mday;	//13;	//UNBCD(RDAYCNT);
    rtcHour = rtc.hour;	//10;	//UNBCD(RHRCNT);
    rtcMin = rtc.min;	//11;	//UNBCD(RMINCNT);
    rtcSec = rtc.sec;	//0;	//UNBCD(RSECCNT);

    tmr = (((DWORD) rtcYear - 80) << 25)
        | ((DWORD) rtcMon << 21)
        | ((DWORD) rtcMday << 16)
        | (WORD) (rtcHour << 11)
        | (WORD) (rtcMin << 5)
        | (WORD) (rtcSec >> 1);

    return tmr;
}

void set_rtc()
{
    DWORD tmr;
    WORD year = rtcYear + 1900;
    /* Real Time Clock */
// RTC STOP
    RYEARCNT = ((BCD((year) / 100) << 8) | BCD((year) % 100));
    RMONCNT = BCD(rtcMon);
    RDAYCNT = BCD(rtcMday);
    RHRCNT = BCD(rtcHour);
    RMINCNT = BCD(rtcMin);
    RSECCNT = BCD(rtcSec);
// RTC START
}

int rtc_gettime (RTCX *rtc)		//BOOL
{
	rtc->sec = RTC_SEC;		//t & 63;
	rtc->min = RTC_MIN;		//(t >> 8) & 63;
	rtc->hour = RTC_HOUR;	//(t >> 16) & 31;
	rtc->wday = RTC_DOW;	//(t >> 24) & 7;
	rtc->mday = RTC_DOM;	//d & 31;
	rtc->month = RTC_MONTH;	//(d >> 8) & 15;
	rtc->year = RTC_YEAR;	//(d >> 16) & 4095;

	return TRUE;
}

int rtc_settime (const RTCX *rtc)			//BOOL
{
	/* Stop RTC */

	/* Update RTC registers */
	RTC_SEC = rtc->sec;
	RTC_MIN = rtc->min;
	RTC_HOUR = rtc->hour;
	RTC_DOW = rtc->wday;
	RTC_DOM = rtc->mday;
	RTC_MONTH = rtc->month;
	RTC_YEAR = rtc->year;

	/* Start RTC with external XTAL */

	return TRUE;
}
